@@ -91,12 +91,16 @@ enum
   PROP_0,
 
   PROP_SURFACE_WIDTH,
-  PROP_SURFACE_HEIGHT
+  PROP_SURFACE_HEIGHT,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 #ifdef CLUTTER_ENABLE_DEBUG
 #define clutter_warn_if_paint_fail(obj)                 G_STMT_START {  \
-  if (CLUTTER_PRIVATE_FLAGS ((obj)) & CLUTTER_ACTOR_IN_PAINT) {         \
+  if (CLUTTER_ACTOR_IN_PAINT (obj)) {                                   \
     g_warning ("%s should not be called during the paint sequence "     \
                "of a ClutterCairoTexture as it will likely cause "      \
                "performance issues.", G_STRFUNC);                       \
@@ -390,11 +394,12 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_uint ("surface-width",
-                             "Surface Width",
-                             "The width of the Cairo surface",
+                             P_("Surface Width"),
+                             P_("The width of the Cairo surface"),
                              0, G_MAXUINT,
                              0,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_SURFACE_WIDTH] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_SURFACE_WIDTH,
                                    pspec);
@@ -407,11 +412,12 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_uint ("surface-height",
-                             "Surface-Height",
-                             "The height of the Cairo surface",
+                             P_("Surface Height"),
+                             P_("The height of the Cairo surface"),
                              0, G_MAXUINT,
                              0,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_SURFACE_HEIGHT] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_SURFACE_HEIGHT,
                                    pspec);
@@ -709,13 +715,13 @@ clutter_cairo_texture_set_surface_size (ClutterCairoTexture *self,
   if (priv->width != width)
     {
       priv->width = width;
-      g_object_notify (G_OBJECT (self), "surface-width");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_SURFACE_WIDTH]);
     }
 
   if (priv->height != height)
     {
       priv->height = height;
-      g_object_notify (G_OBJECT (self), "surface-height");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_SURFACE_HEIGHT]);
     }
 
   clutter_cairo_texture_surface_resize_internal (self);
