@@ -118,9 +118,13 @@ enum
 {
   PROP_0,
   PROP_DURATION,
-  PROP_TARGET_STATE
+  PROP_STATE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   COMPLETED,
@@ -172,9 +176,9 @@ sort_props_func (gconstpointer a,
   return pa->object - pb->object;
 }
 
-static State * clutter_state_get_state (ClutterState *state,
-                                        const gchar  *state_name,
-                                        gboolean      force_creation);
+static State * clutter_state_fetch_state (ClutterState *state,
+                                          const gchar  *state_name,
+                                          gboolean      force_creation);
 static void object_disappeared (gpointer data,
                                 GObject *where_the_object_was);
 
@@ -238,6 +242,8 @@ clutter_state_key_free (gpointer clutter_state_key)
                            key->target_state->clutter_state);
     }
 
+  g_value_unset (&key->value);
+
   g_object_unref (key->alpha);
   g_object_unref (key->interval);
 
@@ -261,7 +267,7 @@ clutter_state_remove_key_internal (ClutterState *this,
   property_name = g_intern_string (property_name);
 
   if (source_state_name)
-    source_state = clutter_state_get_state (this, source_state_name, FALSE);
+    source_state = clutter_state_fetch_state (this, source_state_name, FALSE);
 
   if (target_state_name != NULL)
     state_list = g_list_append (NULL, (gpointer) target_state_name);
@@ -271,27 +277,32 @@ clutter_state_remove_key_internal (ClutterState *this,
   for (s = state_list; s != NULL; s = s->next)
     {
       State *target_state;
-      target_state = clutter_state_get_state (this, s->data, FALSE);
+      target_state = clutter_state_fetch_state (this, s->data, FALSE);
 
       if (target_state)
         {
           GList *k;
-
-          for (k = target_state->keys; k != NULL; k = k?k->next:NULL)
+again:
+          for (k = target_state->keys; k != NULL; k = k->next)
             {
               ClutterStateKey *key = k->data;
 
-              if ((object == NULL || (object == key->object)) &&
-                  (source_state == NULL ||
-                   (source_state == key->source_state)) &&
-                  (property_name == NULL ||
-                   ((property_name == key->property_name))))
+              if (   (object == NULL        || (object == key->object))
+                  && (source_state == NULL  || (source_state == key->source_state))
+                  && (property_name == NULL || ((property_name == key->property_name))))
                 {
-                  k = target_state->keys =
-                    g_list_remove (target_state->keys, key);
+                  target_state->keys = g_list_remove (target_state->keys, key);
+
+                  if (target_state->keys == NULL)
+                    {
+                      /* no more keys, so remove this state */
+                      clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
+                      g_hash_table_remove (this->priv->states, s->data);
+                    }
 
                   key->is_inert = is_inert;
                   clutter_state_key_free (key);
+                  goto again;
                 }
             }
         }
@@ -438,23 +449,7 @@ clutter_state_new_frame (ClutterTimeline *timeline,
 }
 
 
-/**
- * clutter_state_change:
- * @state: a #ClutterState
- * @target_state_name: the state to transition to
- * @animate: whether we should animate the transition or not
- *
- * Change the current state of #ClutterState to @target_state_name
- *
- * If @animate is %FALSE, the state transition will happen immediately;
- * otherwise, the state transition will be animated over the duration
- * set using clutter_state_set_duration()
- *
- * Return value: the #ClutterTimeline that drives the state transition
- *
- * Since: 1.4
- */
-ClutterTimeline *
+static ClutterTimeline *
 clutter_state_change (ClutterState *state,
                       const gchar  *target_state_name,
                       gboolean      animate)
@@ -488,14 +483,14 @@ clutter_state_change (ClutterState *state,
   priv->source_state_name = priv->target_state_name;
   priv->target_state_name = target_state_name;
 
-  g_object_notify (G_OBJECT (state), "target-state");
+  _clutter_notify_by_pspec (G_OBJECT (state), obj_props[PROP_STATE]);
 
   duration = clutter_state_get_duration (state,
                                          priv->source_state_name,
                                          priv->target_state_name);
   clutter_timeline_set_duration (priv->timeline, duration);
 
-  new_state = clutter_state_get_state (state, target_state_name, FALSE);
+  new_state = clutter_state_fetch_state (state, target_state_name, FALSE);
   if (new_state == NULL)
     {
       g_warning ("State '%s' not found", target_state_name);
@@ -556,6 +551,47 @@ clutter_state_change (ClutterState *state,
   return priv->timeline;
 }
 
+/**
+ * clutter_state_set_state:
+ * @state: a #ClutterState
+ * @target_state_name: the state to transition to
+ *
+ * Change the current state of #ClutterState to @target_state_name
+ *
+ * The state will animate during its transition, see
+ * #clutter_state_warp_to_state for animation-free state switching.
+ *
+ * Return value: the #ClutterTimeline that drives the state transition
+ *
+ * Since: 1.4
+ */
+ClutterTimeline *
+clutter_state_set_state (ClutterState *state,
+                         const gchar  *target_state_name)
+{
+  return clutter_state_change (state, target_state_name, TRUE);
+}
+
+/**
+ * clutter_state_warp_to_state:
+ * @state: a #ClutterState
+ * @target_state_name: the state to transition to
+ *
+ * Change the current state of #ClutterState to @target_state_name
+ *
+ * Change to the specified target state immediately with no animation.
+ *
+ * Return value: the #ClutterTimeline that drives the state transition
+ *
+ * Since: 1.4
+ */
+ClutterTimeline *
+clutter_state_warp_to_state (ClutterState *state,
+                             const gchar  *target_state_name)
+{
+  return clutter_state_change (state, target_state_name, FALSE);
+}
+
 static GParamSpec *
 get_property_from_object (GObject     *gobject,
                           const gchar *property_name)
@@ -696,12 +732,15 @@ clutter_state_set (ClutterState *state,
       GParamSpec *pspec;
       GValue value = { 0, };
       gchar *error = NULL;
-      const gchar *real_property_name = property_name;
+      gboolean is_delayed = FALSE;
 
       if (g_str_has_prefix (property_name, "delayed::"))
-        real_property_name = strstr (property_name, "::") + 2;
+        {
+          property_name = strstr (property_name, "::") + 2;
+          is_delayed = TRUE;
+        }
 
-      pspec = get_property_from_object (object, real_property_name);
+      pspec = get_property_from_object (object, property_name);
       if (pspec == NULL)
         break;
 
@@ -721,7 +760,7 @@ clutter_state_set (ClutterState *state,
           break;
         }
 
-      if (g_str_has_prefix (property_name, "delayed::"))
+      if (is_delayed)
         {
           gdouble pre_delay = va_arg (args, gdouble);
           gdouble post_delay = va_arg (args, gdouble);
@@ -730,7 +769,7 @@ clutter_state_set (ClutterState *state,
                                  source_state_name,
                                  target_state_name,
                                  object,
-                                 real_property_name,
+                                 property_name,
                                  mode,
                                  &value,
                                  pre_delay,
@@ -749,6 +788,8 @@ clutter_state_set (ClutterState *state,
                                  0.0);
         }
 
+      g_value_unset (&value);
+
       object = va_arg (args, gpointer);
       if (object != NULL)
         {
@@ -783,25 +824,23 @@ clutter_state_set_key_internal (ClutterState    *state,
 }
 
 /*
- * clutter_state_get_state:
+ * clutter_state_fetch_state:
  * @state: a #ClutterState
  * @state_name: the name of the state to be retrieved
- * @force_creation: %TRUE if the state should be forcibly created
- *   if not found
+ * @create: %TRUE if the state should be instantiated if not found
  *
  * Retrieves the #State structure for @state_name inside the given
  * #ClutterState instance
  *
- * If @state_name is %NULL and @force_creation is %TRUE then the
- * #State for the default state name will be returned; if @force_creation
- * is %FALSE then %NULL will be returned
+ * If @state_name is %NULL and @create is %TRUE then NULL will
+ * be returned.
  *
  * Return value: a #State structure for the given name, or %NULL
  */
 static State *
-clutter_state_get_state (ClutterState *state,
-                         const gchar  *state_name,
-                         gboolean      force_creation)
+clutter_state_fetch_state (ClutterState *state,
+                           const gchar  *state_name,
+                           gboolean      create)
 {
   ClutterStatePrivate *priv = state->priv;
   State *retval;
@@ -814,7 +853,7 @@ clutter_state_get_state (ClutterState *state,
     state_name = g_intern_string (state_name);
 
   retval = g_hash_table_lookup (priv->states, state_name);
-  if (retval == NULL && force_creation)
+  if (retval == NULL && create)
     {
       retval = state_new (state, state_name);
       g_hash_table_insert (priv->states, (gpointer) state_name, retval);
@@ -875,8 +914,8 @@ clutter_state_set_key (ClutterState  *state,
   if (pspec == NULL)
     return state;
 
-  source_state = clutter_state_get_state (state, source_state_name, TRUE);
-  target_state = clutter_state_get_state (state, target_state_name, TRUE);
+  source_state = clutter_state_fetch_state (state, source_state_name, TRUE);
+  target_state = clutter_state_fetch_state (state, target_state_name, TRUE);
 
   property_name = g_intern_string (property_name);
   state_key = clutter_state_key_new (target_state,
@@ -913,7 +952,6 @@ GList *
 clutter_state_get_states (ClutterState *state)
 {
   g_return_val_if_fail (CLUTTER_IS_STATE (state), NULL);
-
   return g_hash_table_get_keys (state->priv->states);
 }
 
@@ -963,13 +1001,13 @@ clutter_state_get_keys (ClutterState *state,
     state_list = clutter_state_get_states (state);
 
   if (source_state_name)
-    source_state = clutter_state_get_state (state, source_state_name, FALSE);
+    source_state = clutter_state_fetch_state (state, source_state_name, FALSE);
 
   for (s = state_list; s != NULL; s = s->next)
     {
       State *target_state;
 
-      target_state = clutter_state_get_state (state, s->data, FALSE);
+      target_state = clutter_state_fetch_state (state, s->data, FALSE);
       if (target_state != NULL)
         {
           GList *k;
@@ -1058,8 +1096,8 @@ clutter_state_set_property (GObject      *object,
 
   switch (prop_id)
     {
-      case PROP_TARGET_STATE:
-        clutter_state_change (state, g_value_get_string (value), TRUE);
+      case PROP_STATE:
+        clutter_state_set_state (state, g_value_get_string (value));
         break;
       case PROP_DURATION:
         state->priv->duration = g_value_get_uint (value);
@@ -1078,9 +1116,9 @@ clutter_state_get_property (GObject    *object,
   ClutterState *state = CLUTTER_STATE (object);
   switch (prop_id)
     {
-      case PROP_TARGET_STATE:
+      case PROP_STATE:
         g_value_set_string (value,
-                            clutter_state_get_target_state (state));
+                            clutter_state_get_state (state));
         break;
       case PROP_DURATION:
         g_value_set_uint (value, state->priv->duration);
@@ -1121,19 +1159,20 @@ clutter_state_class_init (ClutterStateClass *klass)
                   G_TYPE_NONE, 0);
 
   /**
-   * ClutterState:target-state:
+   * ClutterState:state:
    *
    * The currently set target state, setting it causes the
    * state machine to transition to the new state, use
-   * clutter_state_change() directly to directly jump to
-   * a state.
+   * clutter_state_change() with a final FALSE argument to
+   * change state without a transition.
    */
-  pspec = g_param_spec_string ("target-state",
-                               "Target State",
-                               "Currently set state",
+  pspec = g_param_spec_string ("state",
+                               P_("State"),
+                               P_("Currently set state, (transition to this state might not be complete)"),
                                NULL,
                                CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TARGET_STATE, pspec);
+  obj_props[PROP_STATE] = pspec;
+  g_object_class_install_property (gobject_class, PROP_STATE, pspec);
 
   /**
    * ClutterState:duration:
@@ -1142,10 +1181,11 @@ clutter_state_class_init (ClutterStateClass *klass)
    * source/target state pair. The values is in milliseconds.
    */
   pspec = g_param_spec_uint ("duration",
-                             "Duration",
-                             "Default transition duration",
+                             P_("Duration"),
+                             P_("Default transition duration"),
                              0, 86400000, 1000,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 }
 
@@ -1207,7 +1247,7 @@ clutter_state_get_animator (ClutterState *state,
 
   target_state_name = g_intern_string (target_state_name);
 
-  target_state = clutter_state_get_state (state, target_state_name, FALSE);
+  target_state = clutter_state_fetch_state (state, target_state_name, FALSE);
   if (target_state == NULL)
     return NULL;
   
@@ -1258,7 +1298,7 @@ clutter_state_set_animator (ClutterState    *state,
   source_state_name = g_intern_string (source_state_name);
   target_state_name = g_intern_string (target_state_name);
 
-  target_state = clutter_state_get_state (state, target_state_name, FALSE);
+  target_state = clutter_state_fetch_state (state, target_state_name, FALSE);
   if (target_state == NULL)
     return;
   
@@ -1376,20 +1416,26 @@ clutter_state_key_get_mode (const ClutterStateKey *state_key)
  * @state_key: a #ClutterStateKey
  * @value: a #GValue initialized with the correct type for the @state_key
  *
- * Copies of the value for a #ClutterStateKey into @value
+ * Retrieves a copy of the value for a #ClutterStateKey.
  *
  * The #GValue needs to be already initialized for the value type
- * of the property or to a transformable type
+ * of the property or to a type that allow transformation from the value
+ * type of the key.
+ *
+ * Use g_value_unset() when done.
+ *
+ * Return value: %TRUE if the value was successfully retrieved,
+ *   and %FALSE otherwise
  *
  * Since: 1.4
  */
-void
+gboolean
 clutter_state_key_get_value (const ClutterStateKey *state_key,
                              GValue                *value)
 {
-  g_return_if_fail (state_key != NULL);
-  g_return_if_fail (value != NULL);
-  g_return_if_fail (G_VALUE_TYPE (value) != G_TYPE_INVALID);
+  g_return_val_if_fail (state_key != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+  g_return_val_if_fail (G_VALUE_TYPE (value) != G_TYPE_INVALID, FALSE);
 
   if (!g_type_is_a (G_VALUE_TYPE (&state_key->value), G_VALUE_TYPE (value)))
     {
@@ -1397,14 +1443,14 @@ clutter_state_key_get_value (const ClutterStateKey *state_key,
                                    G_VALUE_TYPE (value)))
         {
           g_value_copy (&state_key->value, value);
-          return;
+          return TRUE;
         }
 
       if (g_value_type_transformable (G_VALUE_TYPE (&state_key->value),
                                       G_VALUE_TYPE (value)))
         {
           if (g_value_transform (&state_key->value, value))
-            return;
+            return TRUE;
         }
 
       g_warning ("%s: Unable to convert from %s to %s for the "
@@ -1414,9 +1460,13 @@ clutter_state_key_get_value (const ClutterStateKey *state_key,
                  g_type_name (G_VALUE_TYPE (value)),
                  state_key->property_name,
                  G_OBJECT_TYPE_NAME (state_key->object));
+
+      return FALSE;
     }
   else
     g_value_copy (&state_key->value, value);
+
+  return TRUE;
 }
 
 /**
@@ -1501,6 +1551,28 @@ clutter_state_key_get_target_state_name (const ClutterStateKey *state_key)
   return state_key->target_state->name;
 }
 
+/**
+ * clutter_state_key_get_property_type:
+ * @key: a #ClutterStateKey
+ *
+ * Retrieves the #GType of the property a key applies to
+ *
+ * You can use this type to initialize the #GValue to pass to
+ * clutter_state_key_get_value()
+ *
+ * Return value: the #GType of the property
+ *
+ * Since: 1.4
+ */
+GType
+clutter_state_key_get_property_type (const ClutterStateKey *key)
+{
+  g_return_val_if_fail (key != NULL, G_TYPE_INVALID);
+
+  return G_VALUE_TYPE (&key->value);
+}
+
+
 /**
  * clutter_state_set_duration:
  * @state: a #ClutterState
@@ -1544,7 +1616,7 @@ clutter_state_set_duration (ClutterState *state,
       return;
     }
 
-  target_state = clutter_state_get_state (state, target_state_name, FALSE);
+  target_state = clutter_state_fetch_state (state, target_state_name, FALSE);
   if (target_state != NULL)
     {
       if (source_state_name != NULL)
@@ -1597,7 +1669,7 @@ clutter_state_get_duration (ClutterState *state,
   if (target_state_name == NULL)
     return state->priv->duration;
 
-  target_state = clutter_state_get_state (state, target_state_name, FALSE);
+  target_state = clutter_state_fetch_state (state, target_state_name, FALSE);
   if (target_state != NULL)
     {
       if (source_state_name)
@@ -1615,12 +1687,12 @@ clutter_state_get_duration (ClutterState *state,
 }
 
 /**
- * clutter_state_get_target_state:
+ * clutter_state_get_state:
  * @state: a #ClutterState
  *
  * Queries the currently set target state.
  *
- * During a transition this function will also return the current target.
+ * During a transition this function will return the target of the transition.
  *
  * This function is useful when called from handlers of the
  * #ClutterState::completed signal.
@@ -1631,7 +1703,7 @@ clutter_state_get_duration (ClutterState *state,
  * Since: 1.4
  */
 G_CONST_RETURN gchar *
-clutter_state_get_target_state (ClutterState *state)
+clutter_state_get_state (ClutterState *state)
 {
   g_return_val_if_fail (CLUTTER_IS_STATE (state), NULL);
 
@@ -1659,7 +1731,7 @@ parse_state_transition (JsonArray *array,
   State *source_state, *target_state;
   JsonArray *keys;
   GSList *valid_keys = NULL;
-  GList *k;
+  GList *array_keys, *k;
 
   if (JSON_NODE_TYPE (element) != JSON_NODE_OBJECT)
     {
@@ -1694,10 +1766,10 @@ parse_state_transition (JsonArray *array,
     }
 
   source_name = json_object_get_string_member (object, "source");
-  source_state = clutter_state_get_state (clos->state, source_name, TRUE);
+  source_state = clutter_state_fetch_state (clos->state, source_name, TRUE);
 
   target_name = json_object_get_string_member (object, "target");
-  target_state = clutter_state_get_state (clos->state, target_name, TRUE);
+  target_state = clutter_state_fetch_state (clos->state, target_name, TRUE);
 
   if (json_object_has_member (object, "duration"))
     {
@@ -1731,9 +1803,8 @@ parse_state_transition (JsonArray *array,
   else
     g_value_init (clos->value, G_TYPE_POINTER);
 
-  for (k = json_array_get_elements (keys);
-       k != NULL;
-       k = k->next)
+  array_keys = json_array_get_elements (keys);
+  for (k = array_keys; k != NULL; k = k->next)
     {
       JsonNode *node = k->data;
       JsonArray *key = json_node_get_array (node);
@@ -1811,6 +1882,8 @@ parse_state_transition (JsonArray *array,
       valid_keys = g_slist_prepend (valid_keys, state_key);
     }
 
+  g_list_free (array_keys);
+
   g_value_set_pointer (clos->value, g_slist_reverse (valid_keys));
 
   clos->result = TRUE;
