@@ -31,9 +31,9 @@
 #include "cogl-clip-stack.h"
 #include "cogl-matrix-stack.h"
 #include "cogl-material-private.h"
-#include "cogl-atlas.h"
 #include "cogl-buffer-private.h"
 #include "cogl-bitmask.h"
+#include "cogl-atlas.h"
 
 typedef struct
 {
@@ -56,7 +56,6 @@ typedef struct
 
   /* Enable cache */
   unsigned long     enable_flags;
-  gboolean          fog_enabled;
 
   gboolean          enable_backface_culling;
   CoglFrontWinding  flushed_front_winding;
@@ -73,9 +72,11 @@ typedef struct
   GArray           *texture_units;
   int               active_texture_unit;
 
+  CoglMaterialFogState legacy_fog_state;
+
   /* Materials */
-  CoglHandle        simple_material;
-  CoglHandle	    source_material;
+  CoglMaterial     *simple_material;
+  CoglMaterial     *source_material;
   GString          *arbfp_source_buffer;
 
   int               legacy_state_set;
@@ -93,7 +94,7 @@ typedef struct
   GArray           *polygon_vertices;
 
   /* Some simple caching, to minimize state changes... */
-  CoglHandle	    current_material;
+  CoglMaterial     *current_material;
   unsigned long     current_material_changes_since_flush;
   gboolean          current_material_skip_gl_color;
 
@@ -118,9 +119,9 @@ typedef struct
 
   gboolean              legacy_depth_test_enabled;
 
-  /* PBOs */
-  /* This can be used to check if a pbo is bound */
-  CoglBuffer       *current_pbo;
+  float             point_size_cache;
+
+  CoglBuffer       *current_buffer[COGL_BUFFER_BIND_TARGET_COUNT];
 
   /* Framebuffers */
   GSList           *framebuffer_stack;
@@ -130,7 +131,7 @@ typedef struct
 
   /* Primitives */
   CoglHandle        current_path;
-  CoglHandle        stencil_material;
+  CoglMaterial     *stencil_material;
 
   /* Pre-generated VBOs containing indices to generate GL_TRIANGLES
      out of a vertex array of quads */
@@ -140,10 +141,9 @@ typedef struct
 
   gboolean          in_begin_gl_block;
 
-  CoglHandle        texture_download_material;
+  CoglMaterial     *texture_download_material;
 
   CoglAtlas        *atlas;
-  CoglHandle        atlas_texture;
 
   /* This debugging variable is used to pick a colour for visually
      displaying the quad batches. It needs to be global so that it can
@@ -167,6 +167,10 @@ typedef struct
      cogl_is_texture */
   GSList           *texture_types;
 
+  /* List of types that will be considered a subclass of CoglBuffer in
+     cogl_is_buffer */
+  GSList           *buffer_types;
+
   CoglContextDriver drv;
   CoglContextWinsys winsys;
 } CoglContext;
